#!/usr/bin/env python
import re
from collections import OrderedDict

def readTrimalOut_dict(InfileNameFN):
    Infile = open(InfileNameFN, "r")
    seqDictFN = OrderedDict()
    for Line in Infile:
        Line = Line.rstrip("\n")
        if Line.startswith("    <span class=sel>"):
            name, seq = re.split(r"</span> {2,}", Line)
            name = re.sub(" +<span class=sel>", "", name)
            name = re.sub(" +$", "", name)
            #print("name:", name,"|")
            hitNames = [ele for ele in seqDictFN.keys() if name == ele]
            if not hitNames:
                seqDictFN[name] = ""
            seqDictFN[name] += seq
    Infile.close()
    #for names, values in seqDictFN.items():
    #    print(names)
    #    print(values)
    #exit()
    return seqDictFN


def queryRecPicker(queryNameFN, recsFN):
    keyWord = "^" + queryNameFN + "$"
    hitNames = [ele for ele in recsFN.keys() if re.search(keyWord, ele)]
    if not len(hitNames) == 1:
        print("Stopped. Zero or > 2 hitNames: ", hitNames)
        print("check queryName:", queryNameFN)
        exit()
    queryNameFN = hitNames[0]
    querySeqFN  = recsFN[queryNameFN]
    return queryNameFN, querySeqFN


def trimalOutToSharp(recsFN):
    for name, seq in recsFN.items():
        #print(name)
        seq = re.sub(r"<span class=sel>-</span>",    "-", seq)
        seq = re.sub(r"<span class=sel>X</span>",    "X", seq)
        seq = re.sub(r"<span class=sel>(\w)</span>", "#", seq)
        #print("seq:", seq,"|")
        recsFN[name] = seq
    return recsFN


def compare2seqsUse(querySeq, otherSeq):
    countQuerySeq = 0
    countOtherSeq = 0
    for i in range(len(querySeq)):
        #print(querySeq[i])
        #print(otherSeq[i])
        if querySeq[i] == "#":
            countQuerySeq += 1
            if otherSeq[i] == "#":
                countOtherSeq += 1
        #print(countQuerySeq)
        #print(countOtherSeq)
        #print()
    return round(countOtherSeq/countQuerySeq, 3)


def compare2seqsExtra(querySeq, otherSeq):
    countQuerySeq = 0
    countOtherSeq = 0
    sequenceLength = len(querySeq)
    for i in range(sequenceLength):
        #print(querySeq[i])
        #print(otherSeq[i])
        if querySeq[i] == "#":
            countQuerySeq += 1
        else:
            if re.match("\w", otherSeq[i]) and otherSeq[i]!= "X":
                countOtherSeq += 1
            if otherSeq[i]== "#":
                countOtherSeq += 1
        #print(countQuerySeq)
        #print(countOtherSeq)
        #print()
    return round(countOtherSeq/countQuerySeq, 3)


def rateCalculator(querySeq, recsFN):
    recsUseRateFN = OrderedDict()
    recsExtraRateFN = OrderedDict()
    for name, sequence in recsFN.items():
        #print("name:", name)
        #print("value:", value)

        useRate = compare2seqsUse(querySeq, sequence)
        recsUseRateFN[name] = useRate

        extraRate = compare2seqsExtra(querySeq, sequence)
        recsExtraRateFN[name] = extraRate

    return recsUseRateFN, recsExtraRateFN


## Main program

queryName = "querySeq111"

recs = readTrimalOut_dict("010outfile.html")
recs = trimalOutToSharp(recs)
#for name, seq in recs.items():
#    print(name, seq)
queryName, querySeq = queryRecPicker(queryName, recs)
recsShortRates, recsExtraRates = rateCalculator(querySeq, recs)

print("## ShortRates")
for name, rate in recsShortRates.items():
    print(name,"\t:",rate)
print("## LongRates")
for name, rate in recsExtraRates.items():
    print(name,"\t:",rate)

